aboutsummaryrefslogtreecommitdiffstats
path: root/camel/camel-store.c
diff options
context:
space:
mode:
authorDan Winship <danw@src.gnome.org>2001-01-10 06:25:50 +0800
committerDan Winship <danw@src.gnome.org>2001-01-10 06:25:50 +0800
commit461e7fcb4f4a09d88c750ad61a3fd3011debbc7c (patch)
treeaecdb30ab07914a0572bd28daed2a48dd66fe961 /camel/camel-store.c
parent3c979919b1c8045827767b3e4fa3269cffb87b83 (diff)
downloadgsoc2013-evolution-461e7fcb4f4a09d88c750ad61a3fd3011debbc7c.tar
gsoc2013-evolution-461e7fcb4f4a09d88c750ad61a3fd3011debbc7c.tar.gz
gsoc2013-evolution-461e7fcb4f4a09d88c750ad61a3fd3011debbc7c.tar.bz2
gsoc2013-evolution-461e7fcb4f4a09d88c750ad61a3fd3011debbc7c.tar.lz
gsoc2013-evolution-461e7fcb4f4a09d88c750ad61a3fd3011debbc7c.tar.xz
gsoc2013-evolution-461e7fcb4f4a09d88c750ad61a3fd3011debbc7c.tar.zst
gsoc2013-evolution-461e7fcb4f4a09d88c750ad61a3fd3011debbc7c.zip
Mostly IMAP changes. Use the NAMESPACE extension (where
available). Deal with servers that don't return LIST flags in response to LSUB (like UW) to get rid of the "not a selectable folder" error messages in the UI. Take advantage of the \Marked and \Unmarked flags to try to speed up the folder scan by not doing STATUS on unmarked folders. Some further tweaks on the shape of the resulting folder tree in various situations... * camel-store.h: Remove the (read) message_count, since nothing uses it, and we can speed up IMAP a bit this way. * camel-store.c (camel_folder_info_build): Redo this a bit to make it more useful for IMAP since that's the only thing that uses it. * camel-remote-store.c (camel_remote_store_connected): Public function to check if the store is connected, and try to connect it if it's not. (remote_send_string, remote_send_stream, remote_recv_line): Use that. * providers/imap/camel-imap-store.c (camel_imap_store_finalize): fix up for changes. (camel_imap_store_init): Initialize subscribed_folders to NULL rather than an empty hash table. (imap_connect): Get the list of subscribed folders here. If the server doesn't claim that any of them are either Marked or Unmarked, then assume that it doesn't do that for LSUB and remember that for later. If the server supports the NAMESPACE extension and the user didn't specify a namespace, use the server-provided one. (imap_disconnect): Free the list of subscribed folders, and the namespace. (get_folder): check camel_remote_store_connected (get_folder_info): check camel_remote_store_connected. Add a bunch of new cleverness. If we learned that the server doesn't do LSUB usefully, do a bunch of LISTs by hand. Then, if we're getting unread counts, only do it for folders that weren't listed as Unmarked. Also, deal with namespaces that end with the separator character, and update for changes to camel_folder_info_build. (folder_subscribed): Add a g_return_val_if_fail. (subscribe_folder, unsubscribe_folder): check camel_remote_store_connected. * providers/nntp/camel-nntp-store.c (build_folder_info, build_folder_info_from_grouplist, nntp_store_get_folder_info): Don't fill in message_count since it doesn't exist any more. svn path=/trunk/; revision=7343
Diffstat (limited to 'camel/camel-store.c')
-rw-r--r--camel/camel-store.c72
1 files changed, 44 insertions, 28 deletions
diff --git a/camel/camel-store.c b/camel/camel-store.c
index 48ec13918b..7c61809150 100644
--- a/camel/camel-store.c
+++ b/camel/camel-store.c
@@ -558,48 +558,54 @@ camel_folder_info_free (CamelFolderInfo *fi)
/**
* camel_folder_info_build:
* @folders: an array of CamelFolderInfo
- * @top: the top of the folder tree
+ * @namespace: an ignorable prefix on the folder names
* @separator: the hieararchy separator character
* @short_names: %TRUE if the (short) name of a folder is the part after
* the last @separator in the full name. %FALSE if it is the full name.
*
* This takes an array of folders and attaches them together. @top points
* to the (or at least, "a") top-level element of the tree: it may or may
* not also be an element of @folders. If necessary, camel_folder_info_build
* will create additional CamelFolderInfo with %NULL urls to fill in gaps
* in the tree. The value of @short_names is used in constructing the
* names of these intermediate folders.
**/
-void
-camel_folder_info_build (GPtrArray *folders, CamelFolderInfo *top,
+CamelFolderInfo *
+camel_folder_info_build (GPtrArray *folders, const char *namespace,
char separator, gboolean short_names)
{
- CamelFolderInfo *fi, *pfi;
+ CamelFolderInfo *fi, *pfi, *top = NULL;
GHashTable *hash;
- char *p, *pname;
- int i;
+ char *name, *p, *pname;
+ int i, nlen;
+
+ if (!namespace)
+ namespace = "";
+ nlen = strlen (namespace);
/* Hash the folders. */
hash = g_hash_table_new (g_str_hash, g_str_equal);
- pfi = top;
for (i = 0; i < folders->len; i++) {
fi = folders->pdata[i];
- if (fi == top)
- pfi = NULL;
- g_hash_table_insert (hash, fi->full_name, fi);
+ if (!strncmp (namespace, fi->full_name, nlen))
+ g_hash_table_insert (hash, fi->full_name + nlen, fi);
+ else
+ g_hash_table_insert (hash, fi->full_name, fi);
}
- if (pfi)
- g_hash_table_insert (hash, pfi->full_name, pfi);
/* Now find parents. */
for (i = 0; i < folders->len; i++) {
fi = folders->pdata[i];
- if (fi == top)
- continue;
-
- p = strrchr (fi->full_name, separator);
+ if (!strncmp (namespace, fi->full_name, nlen))
+ name = fi->full_name + nlen;
+ else
+ name = fi->full_name;
+ p = strrchr (name, separator);
if (p) {
- pname = g_strndup (fi->full_name, p - fi->full_name);
+ pname = g_strndup (name, p - name);
pfi = g_hash_table_lookup (hash, pname);
if (pfi) {
g_free (pname);
@@ -620,12 +626,22 @@ camel_folder_info_build (GPtrArray *folders, CamelFolderInfo *top,
fi->sibling = pfi->child;
fi->parent = pfi;
pfi->child = fi;
- } else {
- fi->sibling = top->child;
- fi->parent = top;
- top->child = fi;
- }
+ } else if (!top)
+ top = fi;
}
+ g_hash_table_destroy (hash);
+
+ /* Link together the top-level folders */
+ for (i = 0; i < folders->len; i++) {
+ fi = folders->pdata[i];
+ if (fi->parent || fi == top)
+ continue;
+ if (top)
+ fi->sibling = top;
+ top = fi;
+ }
+
+ return top;
}
gboolean
@@ -558,48 +558,54 @@ camel_folder_info_free (CamelFolderInfo *fi)
* the last @separator in the full name. %FALSE if it is the full name.
*
- * This takes an array of folders and attaches them together. @top points
- * to the (or at least, "a") top-level element of the tree: it may or may
- * not also be an element of @folders. If necessary, camel_folder_info_build
- * will create additional CamelFolderInfo with %NULL urls to fill in gaps
- * in the tree. The value of @short_names is used in constructing the
- * names of these intermediate folders.
+ * This takes an array of folders and attaches them together according
+ * to the hierarchy described by their full_names and @separator. If
+ * @namespace is non-%NULL, then it will be ignored as a full_name
+ * prefix, for purposes of comparison. If necessary,
+ * camel_folder_info_build will create additional CamelFolderInfo with
+ * %NULL urls to fill in gaps in the tree. The value of @short_names
+ * is used in constructing the names of these intermediate folders.
+ *
+ * Return value: the top level of the tree of linked folder info.
**/